#include "Oscilloscope.h"

void Oscilloscope::AddSample(long sample) {
   double array_size_adjust(0.0000005 / ((((double)time_by_div/1000000)*10)/total_samples));
   TPoint point(time++ * array_size_adjust, sample);
   double new_array_size(total_samples/array_size_adjust);
 	NewLine->Add(point);
   if(NewLine->UpperBound() != new_array_size+1) {
   	delete NewLine;
      NewLine = new TPoints(new_array_size+1, 0 ,0);
   	time = 0;
      if(OldLine) {
      	delete OldLine;
         OldLine = 0;
         i_old.Restart();
      }
      parent->Invalidate();
   }
   if(NewLine->GetItemsInContainer() == new_array_size+1) {
      if(!OldLine) {
         OldLine = NewLine;
         i_old = *OldLine;
         NewLine = new TPoints(OldLine->UpperBound(),0 ,0);
      }
      else {
         TPoints* temp(OldLine);
         OldLine = NewLine;
         i_old = *OldLine;
         NewLine = temp;
			NewLine->Flush();
		}
		time = 1;
		TPoint p(0, point.Y());
		NewLine->Add(p);
	}
}

void Oscilloscope::Display(TDC& dc) {
#ifdef PrinterColors
	dc.SetBkColor(TColor::White);
#else
	dc.SetBkColor(TColor::Black);
#endif
// casting to double added by Harry to prevent integer div
	TRect r(parent->Parent->GetClientRect());
	double w((r.Width()-1)/(double)total_samples);
	double h((r.Height()-1)/(double)total_samples);

	/****** paint the background ******/
	dc.SelectObject(whitepen);
	for(int i(1); i<11 ;i++) { 			//paint the vertical division-lines
		dc.MoveTo(((total_samples/(double)10)*w*i), 0);
		dc.LineTo(((total_samples/(double)10)*w*i), total_samples*h);
	}
	for(int i(1); i<11 ;i++) { 			//paint the horizontal division-lines
		dc.MoveTo(0, (total_samples/(double)10)*h*i);
		dc.LineTo(total_samples*w, (total_samples/(double)10)*h*i);
	}

	/****** paint the former tooltip ******/
	// added by Harry
	char buffer[64];
	ostrstream tooltiptext(buffer, sizeof buffer);
	tooltiptext<<GetTimeByDiv()<<" s/div "<<GetVoltsByDiv()<<" mV/div"<<ends;
#ifdef PrinterColors
	dc.SetTextColor(TColor::Black);
#else
	dc.SetTextColor(TColor::LtYellow);
#endif
	dc.SelectObject(TFont("Arial", 16));
	dc.TextOut(10, 5, tooltiptext.str());

	/****** paint the signal ******/
	i_new = *NewLine;
	if(OldLine) {
		while (i_old) {
			TPoint p_old(i_old++);
			dc.SelectObject(blackpen);		//delete the old line
			TPoint scaled_point(p_old.X()*w, (zero_line_pos - p_old.Y() / ((mvolts_by_div*10)/(double)total_samples))*h);
			if(p_old.X() == 0)            //is the first point
				dc.MoveTo(scaled_point);
			dc.LineTo(scaled_point);
		}
	}
	dc.SelectObject(ltgreenpen);		   //paint the new line
	while (i_new) {
		TPoint p_new(i_new++);
		TPoint scaled_point(p_new.X()*w, (zero_line_pos - p_new.Y() / ((mvolts_by_div*10)/(double)total_samples))*h);
		if(p_new.X() == 0)               //is the first point
			dc.MoveTo(scaled_point);
		dc.LineTo(scaled_point);
	}
}

